#include <Arduino.h>
#include <SIM800Bluetooth.hpp>
#include <SIM800HostCommunication.hpp>


SIM800Bluetooth::SIM800Bluetooth() : host()
{
    host.begin(9600);
    
    //Clear btPairDevices struct
    memset(btPairedDevices, 0, sizeof(btPairedDevices));
    
}

bool SIM800Bluetooth::startServer()
{
    return host.checkResponce(host.writeCommand("AT+BTPOWER=1", 1000));
}


bool SIM800Bluetooth::stopServer()
{
    return host.checkResponce(host.writeCommand("AT+BTPOWER=1", 1000));
}

bool SIM800Bluetooth::setHostName(String name)
{
    String command = "AT+BTHOST=" + name + '\r';
    return host.checkResponce(host.writeCommand(command, 1000));
}

bool SIM800Bluetooth::scanDevices(uint8_t time)
{
    //If given time is out of range then scan the default time
    if(time < 10 || time > 60)
        time = 30;
        
    
    host.flush();
    
    String command = "AT+BTSCAN=1," + String(time) + '\r';
    String responce = host.writeCommand(command, 100);
    if(host.checkResponce(responce)){
        //We get OK responce. Scanning is running. Read output untill +BTSCAN: 1 is received
        
        //Remove any characters left in the buffer
        host.flush();
        
        //Empty the struct containing the stanned devices
        memset(btDevices, 0, sizeof(btDevices));
        
        uint8_t numberDevices = 0;
        
        while(true){
            String event = host.readEvent(1000);
            
            #ifdef __DEBUG
            Serial.println(event);
            #endif
            //Check is message is +BTSCAN
            if(event.substring(0, 7) == "+BTSCAN"){
                char ch = event.charAt(9);
                if(ch == '1'){
                    
                    return true;
                    
                }else if(ch == '0'){
                    
                    
                    //Remove "+BTSCAN: 0," from the event message
                    event = event.substring(event.indexOf(',')+1, event.length());
                    
                    //Fill id field
                    //~ Serial.println(event.substring(0, event.indexOf(',')));
                    btDevices[numberDevices].id = event.substring(0, event.indexOf(',')).toInt();
                    event = event.substring(event.indexOf(',')+1, event.length());
                    
                    //Fill name field
                    //~ Serial.println(event.substring(0, event.indexOf(',')));
                    btDevices[numberDevices].name = event.substring(1, event.indexOf(',')-1);
                    event = event.substring(event.indexOf(',')+1, event.length());
                    
                    //Fill address field
                    //~ Serial.println(event);
                    btDevices[numberDevices].address = event;
                    
                    #ifdef __DEBUG__
                    Serial.print("Device num: "); Serial.println(numberDevices);
                    Serial.print("Device id: "); Serial.println(btDevices[numberDevices].id);
                    Serial.print("Device name: "); Serial.println(btDevices[numberDevices].name);
                    Serial.print("Device address: "); Serial.println(btDevices[numberDevices].address);
                    #endif
                    
                    numberDevices++;
                    
                }else if(ch == '2' || ch == '3'){
                    Serial.println(F("Scanning stop or failed"));
                    break;
                }
            }
          
        }
    }else{
        #ifdef __DEBUG
        Serial.println("FAIL: ");
        Serial.println(responce);
        #endif
        return false;
    }
}

bool SIM800Bluetooth::pair(struct _BTDevices device, String passkey)
{
    String responce = host.writeCommand("AT+BTPAIR=0," + String(device.id) + '\r', 1000);
    //~ String responce = host.writeCommand("AT+BTPAIR=0,2\r", 1000);

    
    if(host.checkResponce(responce)){        
        
        //Wait for responce
        while(true){
            String event = host.readEvent(100);          
 
            
            String eventHead = event.substring(0, event.indexOf(':'));
            #ifdef __DEBUG
            Serial.print(F("Eventhead: "));
            Serial.println(eventHead);
            #endif
            if(eventHead == "+BTPAIR"){
        
                char status = event.charAt(9);
                if(status == '0'){
                    return false;                    
                }else{
                    //If char is > 0 then add it to btPairedDevices struct
                    uint8_t i = 0;
                    
                    while(true){
                        if(i >= __MAX_BT_DEVICES__)
                            return false;
                        if(btPairedDevices[i].id == 0){
                            btPairedDevices[i].id = status - '0';
                            btPairedDevices[i].name = device.name;
                            btPairedDevices[i].address = device.address;
                            
                            return true;
                        }else{
                            i++;
                        }
                    }
                }
            }else if(eventHead == "+BTPAIRING"){
                host.writeCommand("AT+BTPAIR=1,1\r", 1000);
            }
        }
    }else{
        #ifdef __DEBUG
        Serial.println(F("Command responce was not OK"));
        Serial.print(F("Responce: "));
        Serial.println(responce);
        #endif
        return false;
    }
}

uint8_t SIM800Bluetooth::countCharsInString(String str, char ch)
{
    uint8_t occ = 0;
    Serial.println("Count");
    Serial.print("String: ");
    Serial.println(str);
    
    for(int i = 0; i < str.length(); i++){
        if(str[i] == ch){
            occ++;
        }
    }
    
    return occ;
}

bool SIM800Bluetooth::unpair(uint8_t id)
{
    return(host.checkResponce(host.writeCommand("AT+BTUNPAIR=" + String(id) + '\r', 1000)));
}

void SIM800Bluetooth::getProfile(uint8_t id)
{
    host.writeCommand("AT+BTGETPROF=" + String(id) + '\r', 100);
    
    while(true){
        String event = host.readEvent(100);
        
        Serial.println(event);
        
    }
}

bool SIM800Bluetooth::connect(uint8_t id, uint8_t profile)
{
    if(host.checkResponce(host.writeCommand("AT+BTCONNECT=" + String(id) + ',' + String(profile) + '\r', 1000))){
        Serial.println("Failed to connect");
        return false;
    }
}
