/*
 * SIM800Bluetooth.hpp>
 * 
 * 
 * Copyright 2014 Stefan Mavrodiev <support@olimex.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */
#ifndef __SIM800Bluetooth_H__
#define __SIM800Bluetooth_H__

#include <SIM800HostCommunication.hpp>

#ifndef __MAX_BT_DEVICES__
#define __MAX_BT_DEVICES__ 5
#endif


struct _BTDevices
{
    uint8_t id;
    String name;
    String address;    
};


class SIM800Bluetooth
{
    private:
        SIM800HostCommunication host;
        
        uint8_t countCharsInString(String str, char ch);
        
    public:
        SIM800Bluetooth();
        bool startServer(); //AT+BTPOWER=1
        bool stopServer();  //AT+BTPOWER=0
        
        bool setHostName(String name);  //AT+BTHOST=<name>
        bool scanDevices(uint8_t time);

        bool pair(struct _BTDevices device, String passkey);
        bool unpair(uint8_t id);
        
        void getProfile(uint8_t id);
        
        bool connect(uint8_t id, uint8_t profile);
        bool disconnect(uint8_t id);
        
        _BTDevices btDevices[__MAX_BT_DEVICES__];
        _BTDevices btPairedDevices[__MAX_BT_DEVICES__];
        
        
};

#endif
