#include "Arduino.h"
#include <SIM800Core.hpp>


SIM800Core::SIM800Core() : host()
{
    
#ifndef __PWRKEY__
#define __PWRKEY__ 11
#endif

#ifndef __RESET__
#define __RESET__ 12
#endif

/*TODO:
 * 1. Check requerements for reset pin
 */
 
    
    pinMode(__PWRKEY__, OUTPUT);
    digitalWrite(__PWRKEY__, LOW);
    
    host.begin(9600);
    //~ delay(100);
    //~ host.echo(false);
}

bool SIM800Core::SWPowerOff(bool isUrgent)
{        
    String poweroffCommand;

#ifdef __DEBUG__
    Serial.println(F(""));
    Serial.println(F("- - - - - - - - - -"));
    Serial.println(F("Function: SWPowerOff"));
    Serial.println(F("- - - - - - - - - -"));
#endif
    
    if(isUrgent)
        poweroffCommand = "AT+CPOWD=0\r";
    else
        poweroffCommand = "AT+CPOWD=1\r";

#ifdef __DEBUG__
    Serial.println(F("Sending command: "));
    Serial.println(poweroffCommand);
#endif

    String commandResponce = host.writeCommand(poweroffCommand, 1000);

#ifdef __DEBUG__
    Serial.println(F("Commmand responce: "));
    Serial.println(commandResponce);
#endif

    //If the poweroff is not urgent, SIM800 will send "NORMAL POWER DOWN".
    String expectedResponce = "NORMAL POWER DOWN";
    
    if(commandResponce == ""){
        if(isUrgent){
#ifdef __DEBUG__
            Serial.println(F("SIM800 powered down urgently"));
#endif
            return true;
        }else{
#ifdef __DEBUG__
            Serial.println(F("SIM800 failed to bring power down"));
#endif
            return false;
        }
    }else{
        //Trim the CR and LF
        commandResponce.trim();
        
        if(commandResponce.length() != expectedResponce.length()){
#ifdef __DEBUG__
            Serial.println(F("Command responce does not match to the expected"));
#endif
            return false;
        }
        
        // Check the command responce
        if(commandResponce.equals(expectedResponce)){
#ifdef __DEBUG__
            Serial.println(F("SIM800 powered down normally"));
#endif
            return true;
        }
    }    
}

bool SIM800Core::HWPowerOn()
{
#ifdef __DEBUG__
    Serial.println(F(""));
    Serial.println(F("- - - - - - - - - -"));
    Serial.println(F("Function: HWPowerOn"));
    Serial.println(F("- - - - - - - - - -"));
#endif
    digitalWrite(__PWRKEY__, HIGH);
    delay(4000);
    digitalWrite(__PWRKEY__, LOW);
    delay(5000);
    
    //Turn off echo
    host.echo(false);
    
    //Flush any incomning data
    host.flush();
    delay(10);
    
    if(!isAlive())
        return false;
    
    //Wait to Call ready and sms ready message
    #ifdef __DEBUG__
    Serial.print("Waiting for Call Ready: ");
    #endif
    while(true){
        String message = host.readEvent(1000);
        if(message.substring(0, 10) == "Call Ready"){
            #ifdef __DEBUG__
            Serial.println("OK");
            #endif
            break;
        }
    }   
    
    //Wait to Call ready and sms ready message
    #ifdef __DEBUG__
    Serial.print("Waiting for SMS Ready: ");
    #endif
    while(true){
        String message = host.readEvent(1000);
        if(message.substring(0, 9) == "SMS Ready"){
            #ifdef __DEBUG__
            Serial.println("OK");
            #endif
            break;
        }
    } 
}

bool SIM800Core::HWPowerOff()
{
#ifdef __DEBUG__
    Serial.println(F(""));
    Serial.println(F("- - - - - - - - - -"));
    Serial.println(F("Function: HWPowerOff"));
    Serial.println(F("- - - - - - - - - -"));
#endif
    digitalWrite(__PWRKEY__, HIGH);
    delay(5000);
    digitalWrite(__PWRKEY__, LOW);
    delay(5000);
    
    host.flush();
    delay(10);
    
    return isAlive();
}

bool SIM800Core::isAlive()
{
#ifdef __DEBUG__
    Serial.print(F("Check if alive? "));
#endif
    //Send test command
    String commandResponce = host.writeCommand("AT\r", 1000);
    
    
    if(commandResponce.equals("")){
#ifdef __DEBUG__
        Serial.println(F("No"));
#endif
        return false;
    }
    
    commandResponce.trim();
    
    if(commandResponce.equals("OK")){
#ifdef __DEBUG__
        Serial.println(F("Yes"));
#endif
        return true;
    }else{
#ifdef __DEBUG__
        Serial.println(commandResponce);
        Serial.println(commandResponce.length());
        Serial.println(F("Yes, but unexpected answer"));
#endif
        return false;
    }
}
