#include <Arduino.h>
#include <SIM800FileSystem.hpp>
#include <SIM800HostCommunication.hpp>



SIM800FileSystem::SIM800FileSystem() : host()
{
    host.begin(9600);
    
    //~ host.flush();
    //~ 
    //~ host.echo(false);
    //~ 
    //~ host.flush();
}

bool SIM800FileSystem::list(String dir)
{     
    
    String responce = host.writeCommand("AT+FSLS=" + dir + '\r', 1000);
    #ifdef __DEBUG__
    Serial.println(responce);
    #endif
    
    
    //Split responce
    while(true){
        String line = "";
        for(uint8_t i = 0; i < responce.length(); i++){
            if(responce[i] == '\n'){
                continue;
            }else if(responce[i] == '\r'){
                if(line == "")
                    continue;
                else{
                    responce = responce.substring(i+1, responce.length());
                    break;
                }
            }else{
                line += responce[i];
            }
        }
        if(line == "OK")
            return true;
        else if(line == "ERROR")
            return false;
        else
            Serial.println(line);
    }
}

bool SIM800FileSystem::createFile(String filename)
{
    return host.checkResponce(host.writeCommand("AT+FSCREATE=" + filename + '\r', 1000));
}

bool SIM800FileSystem::deleteFile(String filename)
{
    return host.checkResponce(host.writeCommand("AT+FSDEL=" + filename + '\r', 1000));
}

int16_t SIM800FileSystem::fileSize(String filename)
{
    host.writeCommand("AT+FSFLSIZE=" + filename + '\r', 0);
    uint16_t size = 0;
    
    while(true){
        String event = host.readEvent(100);
        if(event == "OK"){
            return size;
        }else if(event == "ERROR"){
            return -1;
        }else if(event.substring(0, 9) == "+FSFLSIZE"){
            size = event.substring(event.indexOf(':') + 1, event.length()).toInt();
        }
    }
}

String SIM800FileSystem::readFile(String filename, uint8_t mode, uint16_t filesize, uint16_t position)
{
    String file = "";
    host.writeCommand("AT+FSREAD=" + filename + ',' + String(mode) + ',' + String(filesize) + ',' + String(position) + '\r', 0);
    
    while(true){
        String event = host.readEvent(100);
        if(event == "OK"){
            return file;
        }else if(event == "ERROR"){
            return "";
        }else{
            file += event;
        }
    }
    
}

bool SIM800FileSystem::writeFile(String filename, String text, uint8_t mode)
{
    String responce = host.writeCommand("AT+FSWRITE=" + filename + ',' + String(mode) + ',' + String(text.length()) + ',' + String(2) + '\r', 100);
    
    responce.trim();
    #ifdef __DEBUG__
    Serial.print("Responce: ");
    Serial.println(responce);
    #endif
    
    if(responce == ">"){
        String event = host.writeCommand(text, 2000);
        
        #ifdef __DEBUG__
        Serial.print("Reponce: ");
        event.trim();
        Serial.println(event);
        #endif
        if(event == "OK")
            return true;
        else
            return false;
    }else{
        return false;
    }
}
