#ifndef __SIM800FileSystem_H__
#define __SIM800FileSystem_H__

#include <SIM800HostCommunication.hpp>

class SIM800FileSystem
{
    private:
        SIM800HostCommunication host;
        
    public:
        SIM800FileSystem();
        
        /**
         * List directories/files
         * 
         * @param   <filepath>  String without double quites, directory 
         * for selection. The length of the directory name should be less 
         * than or equal to 64 characters. The string format as 
         * C:\,the directory must have "\"
         * @return  Exist status of the operation
         */
        bool list(String dir);
        
        /**
         * Create new file
         * 
         * @param   <filename>  String without double quotes, the length
         * of the file name should be less than or equal to 64 characters
         * @return  
         */
        bool createFile(String filename);
        
        /**
         * Delete file
         * 
         * @param   <filename>  String without double quites, the file 
         * should have already existed. The length of the file name 
         * should be less than or equal to 64 characters.
         * @return  Exit status of the operation
         */    
        bool deleteFile(String filename);
        
        /**
         * Get file size
         * 
         * @param   <filename>  String without double quotes, the file should
         * have already existed. THe length of the file name should be 
         * less than or equal to 64 characters
         * @return  The file size
         */
        int16_t fileSize(String filename);
        
        /**
         * Read file
         * 
         * @param   filename    String without double quotes, the file 
         * should have already existed. The length of the name should be 
         * less than or equal to 64 characters.
         * @param   mode    0 - Read data at the beginning of the file
         *                  1 - Read data at the <position> of the file
         * @param   filesize    The size of the file that you want to read,
         * the size should be less that or equal to 10240. If the 
         * <filesize> is more than the real size, the <filesize> will be 
         * set to the file real size.
         * @return  String containing the read characters.
         */
        String readFile(String filename, uint8_t mode, uint16_t filesize, uint16_t position);
        
        /**
         * Write file
         * 
         * @param   <filename>  String without double quotes, the file 
         * should have already existed. The length of the file name 
         * should be less than or equal to 64 characters.
         * @param   <text>  String to be written, should be less than 
         * or equal to 10240.
         * @param   <mode>  0 - Write data at the beginning of the file
         *                  1 - Write data at the end of the file
         * @return  Exit status of the operation
         */
        bool writeFile(String filename, String text, uint8_t mode);
        
};
#endif
